#!/usr/bin/perl -w
#
#           d s m N e x t S e r v e r n a m e . p l
#########################################################################
#
# This is a sample TSM for Space Management script for multi-server environments.
# The script demonstrates how the next server for migration and backup
# can be determined from the data that is provided by the output from
# the dsmmigfs querymultiserver command.
#
# The script implements a simple round robin mechanism which just
# returns the next server from the list with each execution.
#
# To utilize the script, the script can be modified as necessary,
# to return the next server based on other information like bytes 
# transferred, files transferred or throughput of the particular server.
#
##########################################################################

# Set binding to HSM commands
$QueryCommand = "/usr/bin/dsmmigfs querymultiserver %s 2>&1 |";

# Set the dsmmigfs header length
$MigfsHeaderLength = 7;

# Validate arg count -- error exit if less than 2 args
die "Usage: $0 Filesystem\n" if ($#ARGV < 0 || $#ARGV > 1);

# First argument is usually the file system name
$filesystem = shift @ARGV;

#########################################################################
#           Read in last used server from file
#########################################################################
$multiServDir = sprintf("%s/.SpaceMan/multiserver", $filesystem);
$lastServerPath = "%s/.SpaceMan/multiserver/.lastServerFile";
$lastServerFile = sprintf($lastServerPath,$filesystem);
$lastUsedServer = "";
$nextUsedServer = "";

# Check if passed file system is valid
opendir(DIR, $multiServDir)
   or die "ERROR: The passed file system $filesystem is not HSM managed or not valid!\n";
closedir(DIR);


# Open the last used server file and read in server name
if (open(SERVERFILE, "<", $lastServerFile))
{	
	while(<SERVERFILE>)
	{
		$lastUsedServer = $_;
	}
	close SERVERFILE;
}

########################################################################### 
#         Execute the query command and pipe output to a file
###########################################################################
$commandoutput = "/tmp/commandoutput.$$";

$syscmd = sprintf($QueryCommand,$filesystem);

# Open the out put file for the query command
open(OUTPUTFILE, ">", $commandoutput)
    or die "$0: Can't open $commandoutput: $!\n";
    
# Execute the query command and print output the the output file
open(QUERYCOMMAND, $syscmd)
    or die "$0: Can't $syscmd: $!\n";
    
    while (<QUERYCOMMAND>) 
    {
        print OUTPUTFILE "$_" ;
    }
    
close QUERYCOMMAND;
close OUTPUTFILE;


############################################################################
#     Parse output from query command and store server specific values
############################################################################
$servername = "";
$bytes = "";
$files = "";
$througput = "";

# Open query output file and store data in a array
open(OUTPUTFILE, "<", $commandoutput)
    or die "$0: Can't open $commandoutput: $!\n";

my @lines = <OUTPUTFILE>;

close OUTPUTFILE;

# Loop over array elements
for($i=0;$i<=$#lines;$i++)
{  
   # Ignore the dsmmigfs command header information
   if ($i > $MigfsHeaderLength)
   {    	  
   	  # Read in servername, bytes transferred, files transferred and the througput
   	  ($servername,$bytes,$files,$througput) = split(/\s+/,$lines[$i]);
   	  
   	  ##################################################
   	  #   Find last used server in server list
   	  ##################################################
   	  
   	  # If last used server was not set yet use current server from list
   	  if ($lastUsedServer eq "")
   	  {
   	  	 $nextUsedServer = $servername;
   	  	 last;
   	  }
   	  else
   	  {
   	  	# If last used server equals the current server from list use next server in list
   	  	if ($lastUsedServer eq $servername)
   	  	{
   	  		$return = $i + 1;
   	  		
   	  		# If return index is bigger then number of elements in array go back to the firts entry
   	  		if ($return > $#lines)
   	  		{
   	  			$return = $MigfsHeaderLength + 1;
   	  		}
   	  		
   	  		# Read server name from line
   	  		($nextUsedServer) = split(/\s+/,$lines[$return]);
   	  		last;
   	  	}
   	  }
   }
}

# if nextUsedServer was not found since lastUsedServer was not in the list anymore 
# set is to last server in the list
if ($nextUsedServer eq "")
{
	$nextUsedServer = $servername;
}

# remove the command output file
$syscmd = "rm $commandoutput";
system($syscmd) == 0
    or warn "$0: Can't $syscmd: $!\n";
    
################################################################################# 
#          Store next server in file and return server name
#################################################################################

# Check for valid server to return
if ($nextUsedServer eq "")
{
	die "ERROR: Server list empty, could not return next server!\n";
}

# Open last used server file and store the new servername
open(SERVERFILE, ">", $lastServerFile)
	or die "ERROR: $0: Can't open $lastServerFile!\n";

# Exclusive lock the server file
flock SERVERFILE, 2;

print SERVERFILE $nextUsedServer;

# Unlock the server file
flock SERVERFILE, 8;

close SERVERFILE;

# Print out new server name 
print "$nextUsedServer\n";
